// Control for HP 6602 ink jet print head
// Details on: https://homofaciens.de/technics-machines-printhead-hp6602_en.htm

#define LED_1       13

#define NOZZLE_01     2
#define NOZZLE_02     3
#define NOZZLE_03     4
#define NOZZLE_04     5
#define NOZZLE_05     6
#define NOZZLE_06     7
#define NOZZLE_07     8
#define NOZZLE_08     9
#define NOZZLE_09    10
#define NOZZLE_10    11
#define NOZZLE_11    12
#define NOZZLE_12    A1

#define COM_1        A4
#define COM_2        A5

#define COMMAND_PAUSE       5
#define DOT_PAUSE_LONG   1000
#define DOT_PAUSE_SHORT     5

int i;
byte readCount;
char comByte = 0;
char dataMode = 0;
long dotWidth = 11;
long lineHeight = 234;
long stepsDone = 0;
char nozzleData[12];

unsigned long nozzle_time[12];

int nozzles[] = {NOZZLE_06, NOZZLE_07, NOZZLE_08, NOZZLE_09, NOZZLE_12, NOZZLE_11, NOZZLE_10, NOZZLE_03, NOZZLE_02, NOZZLE_01, NOZZLE_04, NOZZLE_05};




// Initialize communication through USB
void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');   // send a capital X
    delay(100);
  }
}


void makeDot(int nozzle, long dotPause){
  if(micros() - nozzle_time[nozzle] < DOT_PAUSE_LONG){
     delayMicroseconds(DOT_PAUSE_LONG);
  }
  digitalWrite(nozzle, HIGH);
  // This function works very accurately in the range 3 microseconds and up.
  // We cannot assure that delayMicroseconds will perform precisely for smaller delay-times.
  delayMicroseconds(1);
  digitalWrite(nozzle, LOW);
  delayMicroseconds(dotPause);
  nozzle_time[nozzle] = micros();
}


// the setup routine runs once when you press reset:
void setup(){
  dataMode = 0;
  stepsDone = 0;

  pinMode(LED_1, OUTPUT);

  pinMode(COM_1, INPUT);
  pinMode(COM_2, INPUT);

  for(i=0; i<12; i++){
    pinMode(nozzles[i], OUTPUT);
    nozzle_time[i] = micros();
  }

  // start serial port at 115200 bps:
  Serial.begin(115200);
  establishContact();
  
  readCount = 0;
}

// the loop routine runs over and over again forever:
void loop() {


  if(digitalRead(COM_1) == 1){
    for(int i = 0; i < 12; i++){
      if(digitalRead(COM_2) == 1){
        nozzleData[i] = 1;
      }
      else{
        nozzleData[i] = 0;
      }
      while(digitalRead(COM_1) == 1);
      while(digitalRead(COM_1) == 0);
    }
    for(int i = 0; i < 12; i++){
      if(nozzleData[i] == 1){
        makeDot(i, DOT_PAUSE_SHORT);
      }
    }
  }


}
